local recipes = data.raw.recipe

local recipes_with_items_only = {}

for recipe_name, recipe in pairs (recipes) do
	local is_recipe_with_items_only = true
	local difficulty_table = {}
	if not (recipe.normal) then 
		difficulty_table["single"] = recipe
	else
		difficulty_table["normal"] = recipe.normal
		difficulty_table["expensive"] = recipe.expensive
	end
	for i, u in pairs (difficulty_table) do -- normal, expensive or only the one
		for j, v in pairs (u.ingredients) do
			if not (v.type == "item") then
				is_recipe_with_items_only = false
				log ("Bad ingredients in " .. recipe_name .. ": " .. v.name .. " was " .. v.type .. " (" .. i .. ")")
			end
		end
		for j, v in pairs (u.results) do
			if not (v.type == "item") then
				is_recipe_with_items_only = false
				log ("Bad results in " .. recipe_name .. ": " .. v.name .. " was " .. v.type .. " (" .. i .. ")")
			end
		end
	end
	if is_recipe_with_items_only -- for testing only
	and (data.raw.item[recipe_name]) -- only if it was item recipe; some recipes haven't item :C 
	then
		local k = #recipes_with_items_only+1
		--recipes_with_items_only[k] = recipe_name
		recipes_with_items_only[k] = {name = recipe_name}
		--recipes_with_items_only[k]["name"] = recipe_name
		--recipes_with_items_only[k]["recipe"] = recipe
	end
end

local white_list = {"iron-plate", "iron-gear-wheel", "copper-plate", "copper-cable", "electronic-circuit", "advanced-circuit", "steel-plate", "engine-unit", "low-density-structure",  "rocket-fuel", "rocket-control-unit"}

if white_list then
	recipes_with_items_only = {}
	for i, u in pairs (white_list) do
		recipes_with_items_only[i] = {name = white_list[i]}
	end
end

log ("local recipes_with_items_only = " .. serpent.block(recipes_with_items_only, {comment = false}))

-- deleting all items with entities, test only
for i, recipe in pairs (recipes_with_items_only) do
	local place_result = false
	for item_name, item_v in pairs (data.raw.item) do
		if (recipe.name == item_v.name) then
			if item_v.place_result then
				log ("Deleting: " .. recipe.name .. " was deleted, place_result: " .. item_v.place_result)
				place_result = true
			end	
		end
	end
	if place_result then 
		log ("Deleting: recipes_with_items_only[" .. i .. "] - " .. recipe.name)
		recipes_with_items_only[i] = nil
	end
end

log ("[" .. #recipes_with_items_only .."] local no_place_recipes_with_items_only = " .. serpent.block(recipes_with_items_only, {comment = false}))

local bulk_sizes = {XS = 2, S = 4, M = 12, L = 48, XL = 240}
local bulk_sizes = {XS = 2, S = 4} -- for test


-- making items
local items = {}
local bulk_items = {} -- what is inside, not the bulk
for i, u in pairs (recipes_with_items_only) do
	for bulk_size, bulk_amount in pairs (bulk_sizes) do
		local bulk_item = u.name
		local item_name = "db-" ..bulk_item .."-"..bulk_size
		bulk_items[bulk_item] = bulk_items[bulk_item] or {name = bulk_item, items = {}}
		bulk_items[bulk_item].items[bulk_size] = item_name
		
		local icon = data.raw.item[bulk_item].icon or data.raw.item[bulk_item].icons[1].icon
		local icons = {
			{icon = "__base__/graphics/icons/steel-chest.png"},
			{icon = icon, scale = 0.5},
			{icon = "__DamnedBulks__/graphics/icons/size-" .. bulk_size .. ".png"}
			}

		local item = 
		  {
			 type = "item",
			 name = item_name,
			 icons = icons,
			 flags = {"goes-to-quickbar"},
			 subgroup = "packing-bulks-subgroup",
			 order = i..'-'..bulk_amount,
			 stack_size = math.floor(960/bulk_amount), --50,
			 bulk_item = bulk_item,
			 bulk_size = bulk_size,
			 bulk_amount = bulk_amount
		  }
		items[#items+1] = item
	end
end

log ("[" .. #bulk_items .."] local bulk_items = " .. serpent.block(bulk_items, {comment = false}))

data:extend(items)

-- 'packing my_recipes'
local my_recipes = {}
for i, item in pairs (items) do
	local recipe = 
		{
		 type = "recipe",
		 category = "db-packing",
		 name = "db-packing-" .. item.bulk_item .. "-" .. item.bulk_size,
		 ingredients = {{item.bulk_item, item.bulk_amount}},
		 --icon = "__base__/graphics/icons/uranium-processing.png",
		 results = {{type = item.type, name = item.name, amount = 1}},
		 allow_decomposition = false
		}
	my_recipes[#my_recipes+1] = recipe
end

data:extend(my_recipes)
log ("packing done")

-- 'unpacking recipes'
local my_recipes = {}
for i, item in pairs (items) do
	local recipe = 
		{
		 type = "recipe",
		 category = "db-unpacking",
		 name = "db-unpacking-" .. item.bulk_item .. "-" .. item.bulk_size,
		 ingredients = {{item.name, 1}},
		 icons = item.icons,
		 results = {{item.type, name = item.bulk_item, amount = item.bulk_amount}},
		 group = "bulks-group", -- just for test
		 subgroup = "unpacking-bulks-subgroup",
		 allow_decomposition = false
		}
	my_recipes[#my_recipes+1] = recipe
end

data:extend(my_recipes)
log ("unpacking done")

-- 'recipe replacement'
-- now only one result, sorry
function rnd_from_table (bulk_sizes)
	local i = 0
	local k = 0
	for a, b in pairs (bulk_sizes) do
		k = k + 1
	end
	local rnd = math.random(k)
	for j, v in pairs (bulk_sizes) do
		i = i + 1
		if rnd == i then return j, v end
	end
end

function recipe_results_replacement (recipe_name, recipe, bulk_size)
	local must_be_replaced = nil
	local amount_was = nil
	local rnd_size = rnd_from_table (bulk_sizes)
	
	local bulk_size = bulk_size or "XS"
	for i, res_nr in pairs (recipe.results) do
		for j, v in pairs (recipes_with_items_only) do
			if res_nr.name == v.name then
				must_be_replaced = i
				amount_was = res_nr.amount
			end
		end
	end
	
	for i, num_ing in pairs (recipe.ingredients) do
		for j, item in pairs (items) do
			if (num_ing.name == item.name) or ((num_ing[1] == item.name)) then
				must_be_replaced = false
			end
		end
	end
	
	if must_be_replaced then
		local k = bulk_sizes[bulk_size]
		for i, num_ing in pairs (recipe.ingredients) do
			if not num_ing.amount then
				num_ing.name	 = num_ing[1]
				num_ing.amount	 = num_ing[2]
				num_ing.type = "item"
				num_ing[1] = nil
				num_ing[2] = nil
			end
			--log (i .. " [" .. #num_ing .."] local num_ing = " .. serpent.block(num_ing, {comment = false}))
			num_ing.amount = num_ing.amount * k
		end
		for i, res_nr in pairs (recipe.results) do
			if i == must_be_replaced then
				-- replacing!
				local bulk_item = res_nr.name
				res_nr.name = bulk_items[bulk_item].items[bulk_size]
			else
				res_nr.amount = res_nr.amount * k
			end
		end
		
	end
end

for recipe_name, recipe in pairs (recipes) do -- see first line
	if not (recipe.normal) then 
		recipe_results_replacement (recipe_name, recipe)
	else
		recipe_results_replacement (recipe_name, recipe.normal, "XS")
		recipe_results_replacement (recipe_name, recipe.expensive)
	end
end

data.raw["assembling-machine"]["assembling-machine-1"].crafting_categories[2] = "db-packing"
data.raw["assembling-machine"]["assembling-machine-1"].crafting_categories[3] = "db-unpacking"